#include <stdio.h>
#include <stdlib.h>

struct node {
	int v;
	struct node *next;
};

struct graph {
	int V;
	int E;
	struct node **A;
};

struct graph *creategraph(int nv, int ne)
{
	struct graph *g = malloc(sizeof(struct graph));
	
	if(!g) {
		fprintf(stderr,"Errore di Allocazione\n");
		exit(-1);
	}
	g->E = ne;
	g->V = nv;
	if(!(g->A = calloc(nv,sizeof(struct node *))))  {
		fprintf(stderr,"Errore di Allocazione\n");
		exit(-2);
	}
	return g;
}

struct node *vertexinsert(struct node *p, int k)
{
	struct node *q = malloc(sizeof(struct node));

	if(!q) {
		fprintf(stderr,"Errore di Allocazione\n");
		exit(-3);
	}
	q->v = k;
	q->next = p;
	return q;
}

void readgraph(struct graph *g, FILE *fp)
{
	int i,v1, v2;
	
	for(i = 0; i < g->E; i++) {
		fscanf(fp,"%d %d",&v1,&v2);
		g->A[v1-1] = vertexinsert(g->A[v1-1],v2);
		g->A[v2-1] = vertexinsert(g->A[v2-1],v1);		
	}		
}

void dfs1(struct graph *g, int i, int *aux)
{
	struct node *t;

	aux[i] = 1;
	for(t = g->A[i]; t; t = t->next)
		if(!aux[t->v - 1]) {
			printf("%d,",t->v);
			dfs1(g,t->v-1,aux);
		}
}

void dfs(struct graph *g)
{
	int i;
	int *aux = calloc(g->V,sizeof(int));

	if(!aux) {
		fprintf(stderr,"Errore di Allocazione\n");
		exit(-4);
	}
	for(i = 0; i < g->V; i++)
		if(!aux[i]) {
			printf("\n%d,",i+1);
			dfs1(g,i,aux);
		}
	free(aux);
}

#include "intqueue.h"

void bfs1(struct graph *g, int i, int *aux)
{
	struct node *t;
	intqueue *q = createqueue();
	
	enqueue(q,i);
	while(!emptyq(q)) {
		i = dequeue(q);
		aux[i] = 1;
		for(t = g->A[i]; t; t = t->next)
			if(!aux[t->v - 1]) {
				enqueue(q,t->v -1);
				printf("%d,",t->v);
				aux[t->v-1] = 1;
			}
	}
	destroyqueue(q);
}

void bfs(struct graph *g)
{
	int i;
	int *aux = calloc(g->V,sizeof(int));

	if(!aux) {
		fprintf(stderr,"Errore di Allocazione\n");
		exit(-4);
	}
	for(i = 0; i < g->V; i++)
		if(!aux[i]) {
			printf("\n%d,",i+1);
			bfs1(g,i,aux);
		}
	free(aux);
}

void stampalista(struct node *p)
{
	for(;p; p =p->next)
		printf("->%d",p->v);
}

int main(void)
{
	int i, nv, ne;
	struct graph *g;

	scanf("%d %d", &nv, &ne);
	g = creategraph(nv,ne);
	readgraph(g,stdin);

	for(i = 0; i < nv; i++) {
		printf("\n[%d]",i+1);
		stampalista(g->A[i]);
	}	

	printf("\n\nVisita in profondita':\n");
	dfs(g);

	printf("\n\nVisita in ampiezza:\n");
	bfs(g);
}
