#include "rbtree.h"

rbtree *createrbtree(void)
{
	rbtree *t = malloc(sizeof(rbtree));

	if(!t) {
		fprintf(stderr,"Errore di allocazione A\n");
        	exit(-1);
	}
	if(!(t->root = malloc(sizeof(rbnode)))) {
		fprintf(stderr,"Errore di allocazione B\n");
        	exit(-2);
	}
	t->nil = t->root;
	t->nil->left = t->nil->right = t->nil->up = t->nil;
	t->nil->c = black;
	return t;
}

void inord(rbnode *p, rbnode *nil, void (*op)(rbnode *))
{
	if(p != nil) {
        	inord(p->left,nil,op);
		(*op)(p);
        	inord(p->right,nil,op);
	}
}

void inorder(rbtree *p, void (*op)(rbnode *))
{
	inord(p->root, p->nil, op);
}


rbnode *search(rbtree *r, key k)
{
	rbnode *p = r->root;

	while(p != r->nil && k != p->v)
		p = k < p->v ? p->left : p->right;
	return p == r->nil ? NULL : p;
}

rbnode *rbtmin(rbnode *p, rbnode *nil)
{
	for(;p->left != nil;p = p->left);
	return p;
}

rbnode *rbtmax(rbnode *p, rbnode *nil)
{
	for(;p->right != nil;p = p->right);
	return p;
}

rbnode *treemin(rbtree *r)
{
	return rbtmin(r->root,r->nil);
}

rbnode *treemax(rbtree *r)
{
	return rbtmax(r->root,r->nil);
}

rbnode *treesucc(rbtree *r, rbnode *q)
{
	rbnode *qq;

	if(q->right != r->nil)
		return rbtmin(q->right,r->nil);
	qq = q->up;	
	while(qq != r->nil && q == qq->right) {
		q = qq;
		qq = qq->up;
	}
	return qq == r->nil ? NULL : qq;
}

rbnode *treepred(rbtree *r, rbnode *q)
{
	rbnode *qq;

	if(q->left != r->nil)
		return rbtmax(q->left,r->nil);
	qq = q->up;	
	while(qq != r->nil && q == qq->left) {
		q = qq;
		qq = qq->up;
	}
	return qq == r->nil ? NULL : qq;
}

void leftrotate(rbtree *r, rbnode *x)
{
	rbnode *y = x->right;
	
	x->right = y->left;
	if(y->left != r->nil)
		y->left->up = x;
	y->up = x->up;
	if(x->up == r->nil)
		r->root = y;
	else
		if(x == x->up->left)
			y->up->left = y;
		else
			y->up->right = y;
	y->left = x;
	x->up = y;
}

void rightrotate(rbtree *r, rbnode *x)
{
	rbnode *y = x->left;
	
	x->left = y->right;
	if(y->right != r->nil)
		y->right->up = x;
	y->up = x->up;
	if(x->up == r->nil)
		r->root = y;
	else
		if(x == x->up->right)
			y->up->right = y;
		else
			y->up->left = y;
	y->right = x;
	x->up = y;
}				

rbnode *simpleinsert(rbtree *tree, key k)
{
	rbnode *q = malloc(sizeof(rbnode));
	rbnode *r = tree->root;
	rbnode *s = tree->nil;

	if(!q) { 
		fprintf(stderr,"Errore di allocazione C\n");
        	exit(-4);
	}
	q->v = k;
	q->left = q->right = tree->nil;
	q->c = red;
	while(r != tree->nil) {
		s = r;
		r = k < r->v ? r->left : r->right;
	}
	q->up = s;
	if(s == tree->nil)
		return tree->root = q;
	if(k < s->v)
		s->left = q;
	else
		s->right = q;
	return q;
}

void rbinsert(rbtree *tree, key k)
{
	rbnode *x = simpleinsert(tree, k);
	rbnode *y;

	while(x != tree->root && x->up->c == red) {
		if(x->up == x->up->up->left) {                 /* caso L */
			y = x->up->up->right;
			if(y->c == red) {
				x->up->c = black;                  /* caso 1L */
				y->c = black;                      /* caso 1L */
				x->up->up->c = red;                /* caso 1L */
				x = x->up->up;                     /* caso 1L */
			} else {
				if(x == x->up->right)              /* caso 2L */
					leftrotate(tree,x = x->up);  /* caso 2L */				
				x->up->c = black;                  /* caso 3L */
				x->up->up->c = red;                /* caso 3L */
				rightrotate(tree,x->up->up);       /* caso 3L */
			}
		} else {                                       /* caso R */
			y = x->up->up->left;
			if(y->c == red) {
				x->up->c = black;                  /* caso 1R */
				y->c = black;                      /* caso 1R */
				x->up->up->c = red;                /* caso 1R */
				x = x->up->up;                     /* caso 1R */
			} else {
				if(x == x->up->left)               /* caso 2R */
					rightrotate(tree,x = x->up); /* caso 2R */					
				x->up->c = black;                  /* caso 3R */
				x->up->up->c = red;                /* caso 3R */
				leftrotate(tree,x->up->up);        /* caso 3R */
			}
		}
	}
	tree->root->c = black;
}

void fixup(rbtree *tree, rbnode *x)
{
	rbnode *w;

	while(x != tree->root && x->c == black) {
		if(x == x->up->left) {                                    /* caso L */
			if((w = x->up->right)->c == red) {
				w->c = black;                                 /* caso 1L */
				x->up->c = red;                               /* caso 1L */
				leftrotate(tree,x->up);                       /* caso 1L */
				w = x->up->right;                             /* caso 1L */
			}
			if(w->left->c == black && w->right->c == black) {
				w->c = red;                                   /* caso 2L */
				x = x->up;                                    /* caso 2L */
			} else {
				if(w->right->c == black) {
					w->left->c = black;                     /* caso 3L */
					w->c = red;                             /* caso 3L */
					rightrotate(tree,w);                    /* caso 3L */
					w = x->up->right;                       /* caso 3L */
				}
				w->c = x->up->c;                              /* caso 4L */
				x->up->c = black;                             /* caso 4L */
				w->right->c = black;                          /* caso 4L */
				leftrotate(tree,x->up);                       /* caso 4L */
				x = tree->root;                               /* caso 4L */
			}
		} else {                                                  /* caso R */
			if((w = x->up->left)->c == red) {                   
				w->c = black;                                 /* caso 1R */
				x->up->c = red;                               /* caso 1R */
				rightrotate(tree,x->up);                      /* caso 1R */
				w = x->up->left;                              /* caso 1R */
			}
			if(w->right->c == black && w->left->c == black) {
				w->c = red;                                   /* caso 2R */
				x = x->up;                                    /* caso 2R */
			} else {
				if(w->left->c == black) {
					w->right->c = black;                    /* caso 3R */
					w->c = red;                             /* caso 3R */
					leftrotate(tree,w);                     /* caso 3R */
					w = x->up->left;                        /* caso 3R */
				}
				w->c = x->up->c;                              /* caso 4R */
				x->up->c = black;                             /* caso 4R */
				w->left->c = black;                           /* caso 4R */
				rightrotate(tree,x->up);                      /* caso 4R */
				x = tree->root;                               /* caso 4R */
			}
		}
	}
	x->c = black;
}

void rbdelete(rbtree *tree, rbnode *q)
{
	rbnode *r, *s;

	if(q->left == tree->nil || q->right == tree->nil)
		r = q;
	else
		r = treesucc(tree,q);
	s = r->left != tree->nil ? r->left : r->right;
	s->up = r->up;
	if(r->up == tree->nil)
		tree->root = s;
	else
		if(r == r->up->left)
			r->up->left = s;
		else
			r->up->right = s;
	if(r != q)
		q->v = r->v;
	if(r->c == black)
		fixup(tree, s); 	
	free(r);
}

